local debug = false
if not debug then
    print = function() end
end

local ESX = nil
local QBCore = nil

if Config.Framework == "ESX" then
    ESX = exports["es_extended"]:getSharedObject()
elseif Config.Framework == "QBCORE" then
    QBCore = exports["qb-core"]:GetCoreObject()
end

local function reward(player)
    print(player, Config.BasicReward, Config.MoneyEarned, Config.Framework)
    if Config.BasicReward then
        if Config.Framework == "ESX" then
            local xPlayer = ESX.GetPlayerFromId(player)
            xPlayer.addMoney(Config.MoneyEarned)
            print("ESX", Config.MoneyEarned)
        elseif Config.Framework == "QBCORE" then
            local xPlayer = QBCore.Functions.GetPlayer(player)
            xPlayer.Functions.AddMoney("cash", Config.MoneyEarned, "vote-reward")
            print("QBCORE", Config.MoneyEarned)
        end
        TriggerClientEvent("sp-vote:helpNotification", player, ("Merci de votre Vote ! Récompense : ~g~$%s~s~"):format(Config.MoneyEarned))
    end
end

local players = GetPlayers()
local processedVotes = {}

local function IsVoteProcessed(vote)
    local diff = os.time() - vote.voted_at
    for _, processedVote in ipairs(processedVotes) do
        if (processedVote.voted_at == vote.voted_at and processedVote.username == vote.username) or diff > 5400 then
            return true
        end
    end
    return false
end

local function verifRpNames(names)
    print("verifRpNames")
    if Config.Framework == "ESX" then
        print("ESX")
        MySQL.Async.fetchAll("SELECT * FROM users WHERE CONCAT(firstname, ' ', lastname) = @names", {
            ["@names"] = names
        }, function(result)
            if result[1] ~= nil then
                return true
            else
                return false
            end
        end)
    elseif Config.Framework == "QBCORE" then
        print("QBCORE")
        MySQL.Async.fetchAll("SELECT * FROM players", {}, function(result)
            for k, v in pairs(result) do
                local charinfo = json.decode(v.charinfo)
                if string.format("%s %s", charinfo.firstname:lower(), charinfo.lastname:lower()) == names then
                    return true
                else
                    return false
                end
            end
        end)
    end
end

local function ProcessVote(vote)
    print("ProcessVote")
    local playerFound = false
    players = GetPlayers()
    if #players > 0 then
        for k, v in pairs(players) do
            local playerName = GetPlayerName(tonumber(v))
            if (vote.username:lower() == playerName:lower()) then
                print("Players online")
                TriggerEvent("sp-vote:newVoteOnline", tonumber(v))
                print(("New Vote Online - Voted at: %s - Username: %s"):format(vote.voted_at, vote.username))
                playerFound = true
                break
            end
        end
        if verifRpNames(vote.username:lower()) then
            playerFound = true
        end
    end
    if not playerFound then
        print("Players ofline")
        TriggerEvent("sp-vote:newVoteOffline", vote)
    end
end

local function FetchVotes()
    print("FetchVotes")
    PerformHttpRequest(("https://serveur-prive.net/api/v1/servers/%s/votes"):format(Config.Token), function(errorCode, resultData, resultHeaders)
        print("PerformHttpRequest", errorCode, resultData, resultHeaders)
        if errorCode ~= nil and errorCode == 200 then
            if resultData ~= nil then
                local resultData = json.decode(resultData)
                if type(resultData) == "table" and type(resultData.data) == "table" then
                    for _, vote in ipairs(resultData.data) do
                        if type(vote) == "table" and type(vote.voted_at) == "number" and type(vote.username) == "string" then
                            print(("Vote detect - Voted at: %s - Username: %s"):format(vote.voted_at, vote.username))
                            if not IsVoteProcessed(vote) then
                                ProcessVote(vote)
                                table.insert(processedVotes, vote)
                            end
                        end
                    end
                end
            end
        end
    end, "GET")
end

local function VotesUpdater()
    FetchVotes()
    SetTimeout(30000, VotesUpdater)
end

Citizen.CreateThread(function()
    VotesUpdater()
end)

RegisterServerEvent("sp-vote:newVoteOnline")
AddEventHandler("sp-vote:newVoteOnline", function(player)
    reward(player)
end)

RegisterServerEvent("sp-vote:newVoteOffline")
AddEventHandler("sp-vote:newVoteOffline", function(vote)
    print("sp-vote:newVoteOffline")
    MySQL.Async.execute("INSERT INTO sp_votes (voted_at, username) VALUES (@voted_at, @username)", {
        ["@voted_at"] = vote.voted_at,
        ["@username"] = vote.username
    }, function(rowsChanged)
        print(("New Vote Offline - Voted at: %s - Username: %s"):format(vote.voted_at, vote.username))
    end)
end)

RegisterServerEvent("sp-vote:checkVote")
AddEventHandler("sp-vote:checkVote", function()
    print("sp-vote:checkVote")
    local _source = source
    local playerName = GetPlayerName(_source)
    MySQL.Async.fetchAll("SELECT * FROM sp_votes WHERE username = @username", {
        ["@username"] = playerName
    }, function(result)
        if result[1] ~= nil then
            TriggerClientEvent("sp-vote:helpNotification", _source, ("Merci de votre Vote ! Récompense : ~g~$%s~s~"):format(Config.MoneyEarned))
            reward(_source)
            MySQL.Async.execute("DELETE FROM sp_votes WHERE username = @username", {
                ["@username"] = playerName
            }, function(rowsChanged)
                print(("Vote supprimé - Username: %s"):format(playerName))
            end)
        else
            print("not cfx name", playerName:lower())
            if verifRpNames(playerName:lower()) then
                TriggerClientEvent("sp-vote:helpNotification", _source, ("Merci de votre Vote ! Récompense : ~g~$%s~s~"):format(Config.MoneyEarned))
                reward(_source)
            else
                TriggerClientEvent("sp-vote:helpNotification", _source, "Vous n'avez pas encore voté ! Votez sur : serveur-prive.net")
            end
        end
    end)
end)
